#property copyright "Public domain software by Ralph Ronnquist"
#property link      ""

// This EA enters trades only once a day, and nurtures them by using "hidden" takeprofit
// and stoploss. The entry logic is "break out", with a test that relates current price
// to the price at a certain "sample hour". If current price is beyond a certain range
// from the sample price, then a position in the same direction is entered. The test range
// as well as TP and SL are obtained by a history review, that uses the price movements of
// a few 24-hour periods preceding the sample hour. 
//
// Parameters:
// timezone = set to broker's time zone
// magic = set to something
// SAMPLE_HOUR = the time that defines sample price
// EXPIRES = how many hours following sample time this EA should maybe open trades
// CLOSE_HOUR = at which hour trades should be closed, regardless (use -1 for never)
// RATE = a least number of seconds between EA activity
// METHOD = selection of history review method:
//    1 = use average ranges of the last five 24-hour periods
//    2 = use average of the extreme ranges of the last ten 25-hour periods
//    3 = (default) use median range of the last few 24-hour periods
// Rop = percentage of review range defining entry threshold
// Rtp = percentage of review range defining takeprofit
// Rsl = percentage of review range defining stoploss
// lots = volume of each trade
//-----------------------------------------------------------------------------------
extern int timezone=0;
extern int magic = 96;
extern int SAMPLE_HOUR = 7;
extern int EXPIRES = 23;
extern int CLOSE_HOUR = 6;
extern int RATE = 1;
extern int METHOD = 2;
extern int Rop = 25; // Percentage of range for OP
extern int Rtp = 75; // Percentage of range for TP
extern int Rsl = 25; // Percentage of range for SL
extern double lots = 1.0;

//********** EA Initialisation
// This function is called when the EA is started or re-started (with new parameters)
int init()
{
    if ( RATE < 1 ) RATE = 1;
    return( 0 );
}

//********** EA Termination
// This function is called when the EA is terminated
int deinit()
{
    return( 0 );
}

//********** EA Operation
// This function is called once when new tick data has been received for the symbol that
// the EA is attached to.
int start()
{
    // Only operate once every RATE seconds
    static datetime last_active = 0;
    int N = TimeCurrent() / RATE;
    if ( N <= last_active ) return( 0 );
    last_active = N;

    // Find the applicable trade measures by reviewing recent price movements, which
    // is summarised in a "range" measure. For performance reasons, these measures are
    // only adjusted each day.
    static int OP;
    static int TP;
    static int SL;
    static int day = -1;
    if ( day != TimeDay( TimeCurrent() ) ) {
        int range = sampleRange();
        OP = range * ( 0.01 * Rop );
        TP = range * ( 0.01 * Rtp );
        SL = range * ( 0.01 * Rsl );
        day = TimeDay( TimeCurrent() );
    }
    
    // Determine the date stamp for the sample
    // gmt_day is the current date at GMT time
    // sample_time is the time stamp the broker has for the selected GMT sample hour
    string gmt_day = TimeToStr( TimeCurrent() - timezone * 3600, TIME_DATE );
    datetime sample_time = StrToTime( gmt_day ) + ( SAMPLE_HOUR + timezone ) * 3600 ;
    
    // Avoid trading at today's sample time if  it is before the very first EA tick time
    static datetime first_tick = 0;
    if ( first_tick == 0 ) first_tick = TimeCurrent();
    if ( sample_time < first_tick ) return( 0 );

    // last_sample is the datetime for the most recent sample time
    datetime last_sample = sample_time;
    if ( last_sample > TimeCurrent() )
        last_sample -= 86400;
    
    // Review open trades in order to apply the closing condition. It also determines
    // whether there are any open trades, in which case no more trades should be opened.
    int i;
    double price;
    bool some_open = false;
    int h = TimeHour( TimeCurrent() ) - timezone;
    if ( h < 0 ) h += 24;
   
    //-- Calculating the highest high and lowest low value since most recent sample hour. 
    int SB = iBarShift( Symbol(), PERIOD_M5, last_sample, true ); 
    double HHV = High[ iHighest( Symbol(), PERIOD_M5, MODE_HIGH, SB, 0 ) ];  
    double LLV = Low[ iLowest( Symbol(), PERIOD_M5, MODE_LOW, SB, 0 ) ]; 
    
    //-- Sample price = open price at most recent sample hour
    double sample_price = iOpen( Symbol(), PERIOD_M5, SB );

    //-- Calculations for trailing stop: when price has moved more than 60% of target,
    // then stop should be at 80% of that peak (or trough).
    int MaxLongProfitSoFar = ( HHV - sample_price ) / Point - OP;  // peak, in pips
    int MaxShortProfitSoFar = ( sample_price - LLV ) / Point - OP; // trough, in pips
    int MaxProfitThreshold = 0.6 * ( TP - OP ); // the threshold

    double move; // ... to hold the current price move (positive), in pips * Point
    double take = TP * Point;
    double stop = - ( OP + SL ) * Point ; // Note that this is < 0
       
    for ( i = OrdersTotal() - 1; i >=0; i-- ) {
        if ( ! OrderSelect( i, SELECT_BY_POS, MODE_TRADES ) )
            continue;
        if ( magic != 0 && OrderMagicNumber() != magic )
            continue;
        if ( OrderSymbol() != Symbol() )
            continue;
        if ( OrderType() == OP_BUY ) {
            if ( h != CLOSE_HOUR ) {
                move = Bid - ( sample_price + OP * Point ); // Positive upwards
                take = LLV - sample_price + ( TP - OP ) * Point; // Moving takeprofit
                if ( MaxLongProfitSoFar >= MaxProfitThreshold ) {
                    stop = 0.8 * MaxLongProfitSoFar * Point;
                }
                if ( move < take && move > stop ) {
                    some_open = true;
                    continue; // Avoids closing this trade
                }
            }
            price = Bid;
        } else if ( OrderType() == OP_SELL ) {
            if ( h != CLOSE_HOUR ) {
                move = ( sample_price - OP * Point ) - Ask; // Positive downwards
                take = sample_price - HHV + ( TP - OP ) * Point;
                if ( MaxShortProfitSoFar >= MaxProfitThreshold ) {
                    stop = 0.8 * MaxShortProfitSoFar * Point;
                }
                if ( move < take && move > stop ) {
                    some_open = true;
                    continue; // Avoids closing this trade
                }
            }
            price = Ask;
        }
        if ( ! OrderClose( OrderTicket(), OrderLots(), price, Yellow ) )
            some_open = true;
    }
    
     // Avoid opening new trade if there is one open
    if ( some_open )
        return( 0 );


    // Review account history for a trade after the sample time, so as to avoid
    // a second one.
    for ( i = OrdersHistoryTotal() - 1; i >=0; i-- ) {
        if ( ! OrderSelect( i, SELECT_BY_POS, MODE_HISTORY ) )
            continue;
        if ( magic != 0 && OrderMagicNumber() != magic )
            continue;
        if ( OrderSymbol() != Symbol() )
            continue;
        if ( OrderOpenTime() >= last_sample ) {
            return( 0 );
        }
    }
  
    // Limit the enabling period to be EXPIRES hours after sample SAMPLE_HOUR each day.
    if ( TimeCurrent() < sample_time ) return( 0 ); // Too early
    if ( TimeCurrent() >= sample_time + EXPIRES * 3600 ) return( 0 ); // Too late
 
    // Check for opening condition

    int type = -1;
    if ( Ask >= sample_price + OP * Point ) {
        type = OP_BUY;
        price = Close[0];
    } else if ( Bid <= sample_price - OP * Point  ) {
        type = OP_SELL;
        price = Close[0];
    } else {
        return( 0 );
    }
    OrderSend( Symbol(), type, lots, price, 0, 0, 0, "", magic, 0, CLR_NONE );
    return(0);
}

////// Extra functions

// Reviews recent price movement to determines a base-line range measure for the
// trading variables. It applies the selected one of three methods:
//   1 = use the average of the ranges of the last five 24-hour periods
//   2 = use the average of the two extreme ranges of the last ten 24-hour periods
//   3 = use the median of the ranges of the last few 24-hour periods
// In all cases, it's the movements before the SAMPLE_HOUR that is used.
int sampleRange()
{
    int bar = 0;
    int bars = iBars( Symbol(), PERIOD_H1 );
    for ( bar = 0; bar < bars; bar++ ) {
        int h = TimeHour( iTime( Symbol(), PERIOD_H1, bar ) ) - timezone;
        if ( h < 0 ) h += 24;
        if ( h == SAMPLE_HOUR ) break;
    }
    if ( bar >= bars ) return( 0 );

    switch ( METHOD ) {
    case 1: return( rangeAverage( bar ) );       // by TheHedge
    case 2: return( rangeMinMaxAverage( bar ) ); // by Learner
    default: return( rangeMedian( bar ) );       // by my mistake
    }
    // ** returned before here **
}

//*** The function is used in the TheHedge variant.
// Computes the moving average of the last five 24-hour periods prior to
// the given H1 bar.
int rangeAverage(int bar)
{
    int sum = 0;
    for ( int i = 0; i < 5; i++ ) {
        sum += range( bar );
    }
    return( sum / 5 );
}        

//*** The function is used in the Learner variant.
// Computes the average of the two extreme ranges of the last ten 24-hour periods
// prior to the given H1 bar.
int rangeMinMaxAverage(int bar)
{
    int least = range( bar );
    int most = least;
    for ( int i = 0; i < 9; i++ ) {
        int r = range( bar );
        least = MathMin( least, r );
        most = MathMax( most, r );
    }
    return( ( least + most ) / 2 );
}

//*** The function is used in my mistaken Learner variant.
// Computes the median range of the last EXTENT 24-hour periods prior to
// the given H1 bar.
#define EXTENT 5
int rangeMedian(int bar)
{
    double ranges[ EXTENT ];
    for ( int i = 0; i < EXTENT; i++ ) {
        ranges[ i ] = range( bar );
    }
    ArraySort( ranges );
    int mid = EXTENT / 2;
    if ( ( EXTENT & 1 ) == 1 )
        return( ranges[ mid ] ); // Odd sized extent
    return( ( ranges[ mid ] + ranges[ mid+1 ] ) / 2 ); // Even sized extent
}

// Computes the price range of the 24-hour period prior to the given H1 bar,
// and it also moves that index to the beginning of that 24-hour period.
// Note: doesn't handle missing bars very well.
int range(int &bar)
{
    int h = TimeHour( iTime( Symbol(), PERIOD_H1, bar ) );
    double hi = 0;
    double lo = 10000;
    if ( bar + 1 >= Bars ) {
        bar = Bars;
        return( 0 );
    }
    for ( bar = bar + 1; bar < Bars; bar++ ) {
        hi = MathMax( hi, iHigh( Symbol(), PERIOD_H1, bar ) );
        lo = MathMin( lo, iLow( Symbol(), PERIOD_H1, bar  ) );
        if ( h == TimeHour( iTime( Symbol(), PERIOD_H1, bar ) ) )
            break;
    }
    return( ( hi - lo ) / Point );
}

